#include "PhysicsComponent.h"
#include "../Entitys/Object.h"
#include "imgui.h"

PhysicsComponent::PhysicsComponent(BodyType type, float width, float height)
    : type(type), width(width), height(height) {}

PhysicsComponent::~PhysicsComponent() {
    DestroyBody();
}

void PhysicsComponent::CreateBody(b2World* world, const glm::vec2& position) {
    this->world = world;
    b2BodyDef def;

    switch (type) {
        case BodyType::Static:    def.type = b2_staticBody; break;
        case BodyType::Dynamic:   def.type = b2_dynamicBody; break;
        case BodyType::Kinematic: def.type = b2_kinematicBody; break;
    }

    def.position.Set(position.x, position.y);
    def.fixedRotation = fixedRotation;

    body = world->CreateBody(&def);

    b2PolygonShape box;
    box.SetAsBox(width * 0.5f, height * 0.5f);

    b2FixtureDef fixtureDef;
    fixtureDef.shape = &box;
    fixtureDef.density = density;
    fixtureDef.friction = friction;

    body->CreateFixture(&fixtureDef);
}

void PhysicsComponent::DestroyBody() {
    if (body && world) {
        world->DestroyBody(body);
        body = nullptr;
    }
}

void PhysicsComponent::SetVelocity(const glm::vec2& vel) {
    if (body) body->SetLinearVelocity(b2Vec2(vel.x, vel.y));
}

glm::vec2 PhysicsComponent::GetVelocity() const {
    if (body) {
        b2Vec2 v = body->GetLinearVelocity();
        return { v.x, v.y };
    }
    return {};
}

void PhysicsComponent::SetFriction(float f) {
    friction = f;
    if (body && body->GetFixtureList())
        body->GetFixtureList()->SetFriction(f);
}

void PhysicsComponent::SetDensity(float d) {
    density = d;
    // requires refixture for update
}

void PhysicsComponent::SetFixedRotation(bool fixed) {
    fixedRotation = fixed;
    if (body)
        body->SetFixedRotation(fixed);
}

void PhysicsComponent::SyncFromPhysics() {
    if (body) {
        b2Vec2 pos = body->GetPosition();
        object->SetLocalPosition({ pos.x, pos.y });
    }
}

void PhysicsComponent::SyncToPhysics() {
    if (body) {
        glm::vec2 pos = object->GetLocalPosition();
        body->SetTransform({ pos.x, pos.y }, body->GetAngle());
    }
}

void PhysicsComponent::OnImGuiInspector() {
    ImGui::Text("Physics Body");
    glm::vec2 vel = GetVelocity();
    ImGui::DragFloat2("Velocity", &vel.x);
    SetVelocity(vel);

    if (ImGui::DragFloat("Friction", &friction, 0.01f, 0.0f, 1.0f))
        SetFriction(friction);

    if (ImGui::Checkbox("Fixed Rotation", &fixedRotation))
        SetFixedRotation(fixedRotation);
}
